Option Explicit On
Option Strict On

Imports System.Math

Module MathGammaF
    Public Function GammLn(ByVal X As Double) As Double
        If X <= 0.0 Then Throw New System.ArgumentOutOfRangeException _
                                    ("X", X, "The argument must be positive.")

        '---------------------------------------
        'Based on Oura's Expansion and Constants
        '---------------------------------------
        Const myu As Double = 6.0975075753906856
        Const ainf As Double = 0.0056360656189756069
        Const a0 As Double = 0.12242597732687992
        Const a1 As Double = 0.85137081316503416
        Const a2 As Double = 2.2502304753561817
        Const a3 As Double = 2.09629553538955
        Const a4 As Double = 0.50219722703392089
        Const a5 As Double = 0.011240582657165408
        Const b1 As Double = 1.0000000000006553
        Const b2 As Double = 1.9999999996201023
        Const b3 As Double = 3.000000046726524
        Const b4 As Double = 3.999996630000751
        Const b5 As Double = 5.0003589884831925

        Dim temp As Double = a5 / (X + b5) + a4
        temp = temp / (X + b4) + a3
        temp = temp / (X + b3) + a2
        temp = temp / (X + b2) + a1
        temp = temp / (X + b1) + a0
        temp = temp / X + ainf
        Return Log(temp) + (X - 0.5) * Log(X + myu) - X

        '--------------------------
        'Based on NUMERICAL RECIPES
        '--------------------------
        'Const cof0 As Double = 76.180091729471457
        'Const cof1 As Double = -86.505320329416776
        'Const cof2 As Double = 24.014098240830911
        'Const cof3 As Double = -1.231739572450155
        'Const cof4 As Double = 0.001208650973866179
        'Const cof5 As Double = -0.000005395239384953

        'Dim temp As Double = X + 5.5
        'temp -= (X + 0.5) * Log(temp)

        'Dim ser As Double = 1.0000000001900149
        'ser += cof0 / (X + 1.0)
        'ser += cof1 / (X + 2.0)
        'ser += cof2 / (X + 3.0)
        'ser += cof3 / (X + 4.0)
        'ser += cof4 / (X + 5.0)
        'ser += cof5 / (X + 6.0)

        'Return -temp + Log(2.5066282746310007 * ser / X)
    End Function
    Public Function FactRl(ByVal n As Integer) As Double
        Static ntop As Integer = 4
        Static a() As Double = {1.0, 1.0, 2.0, 6.0, 24.0, 0.0, 0.0, 0.0, _
                                0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, _
                                0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, _
                                0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}

        If n < 0 Then Throw New System.ArgumentOutOfRangeException _
                                    ("n", n, "The argument must be zero or positive.")
        If n > 32 Then Return Exp(GammLn(n + 1.0))
        Do While ntop < n
            Dim j As Integer = ntop
            ntop += 1
            a(ntop) = a(j) * ntop
        Loop
        Return a(n)
    End Function
    Public Function FactLn(ByVal n As Integer) As Double
        Static a(100) As Double

        If n < 0 Then Throw New System.ArgumentOutOfRangeException _
                                    ("n", n, "The argument must be zero or positive.")
        If n <= 1 Then Return 0.0
        If n <= 100 Then
            If a(n) = 0.0 Then a(n) = GammLn(n + 1.0)
            Return a(n)
        Else
            Return GammLn(n + 1.0)
        End If
    End Function
    Public Function BiCo(ByVal n As Integer, ByVal k As Integer) As Double
        Return Floor(0.5 + Exp(FactLn(n) - FactLn(k) - FactLn(n - k)))
    End Function
    Public Function Beta(ByVal z As Double, ByVal w As Double) As Double
        Return Exp(GammLn(z) + GammLn(w) - GammLn(z + w))
    End Function
End Module
